@@ -15,6 +15,8 @@
 #               9) http://www.w3.org/TR/SVG11/
 #               11) http://www.extensis.com/en/support/kb_article.jsp?articleNumber=6102211
 #               12) XMPSpecificationPart3_May2013, page 58
+#               13) https://developer.android.com/training/camera2/Dynamic-depth-v1.0.pdf
+#               14) http://www.iptc.org/standards/photo-metadata/iptc-standard/
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::XMP;
@@ -76,6 +78,65 @@ my %sTimecode = (
     timeValue   => { },
     value       => { Writable => 'integer', Notes => 'only in XMP 2008 spec; an error?' },
 );
+my %sPose = (
+    STRUCT_NAME => 'Pose',
+    NAMESPACE => { Pose => 'http://ns.google.com/photos/dd/1.0/pose/' },
+    PositionX => { Writable => 'real', Groups => { 2 => 'Location' } },
+    PositionY => { Writable => 'real', Groups => { 2 => 'Location' } },
+    PositionZ => { Writable => 'real', Groups => { 2 => 'Location' } },
+    RotationX => { Writable => 'real', Groups => { 2 => 'Location' } },
+    RotationY => { Writable => 'real', Groups => { 2 => 'Location' } },
+    RotationZ => { Writable => 'real', Groups => { 2 => 'Location' } },
+    RotationW => { Writable => 'real', Groups => { 2 => 'Location' } },
+    Timestamp => {
+        Writable => 'integer',
+        Shift => 'Time',
+        Groups => { 2 => 'Time' },
+        ValueConv => 'ConvertUnixTime($val / 1000, 1, 3)',
+        ValueConvInv => 'int(GetUnixTime($val, 1) * 1000)',
+        PrintConv => '$self->ConvertDateTime($val)',
+        PrintConvInv => '$self->InverseDateTime($val,undef,1)',
+    },
+);
+my %sEarthPose = (
+    STRUCT_NAME => 'EarthPose',
+    NAMESPACE => { EarthPose => 'http://ns.google.com/photos/dd/1.0/earthpose/' },
+    Latitude  => { Writable => 'real', Groups => { 2 => 'Location' }, %latConv },
+    Longitude => { Writable => 'real', Groups => { 2 => 'Location' }, %longConv },
+    Altitude  => {
+        Writable => 'real',
+        Groups => { 2 => 'Location' },
+        PrintConv => '"$val m"',
+        PrintConvInv => '$val=~s/\s*m$//;$val',
+    },
+    RotationX => { Writable => 'real', Groups => { 2 => 'Location' } },
+    RotationY => { Writable => 'real', Groups => { 2 => 'Location' } },
+    RotationZ => { Writable => 'real', Groups => { 2 => 'Location' } },
+    RotationW => { Writable => 'real', Groups => { 2 => 'Location' } },
+    Timestamp => {
+        Writable => 'integer',
+        Shift => 'Time',
+        Groups => { 2 => 'Time' },
+        ValueConv => 'ConvertUnixTime($val / 1000, 1, 3)',
+        ValueConvInv => 'int(GetUnixTime($val, 1) * 1000)',
+        PrintConv => '$self->ConvertDateTime($val)',
+        PrintConvInv => '$self->InverseDateTime($val,undef,1)',
+    },
+);
+my %sVendorInfo = (
+    STRUCT_NAME => 'VendorInfo',
+    NAMESPACE   => { VendorInfo => 'http://ns.google.com/photos/dd/1.0/vendorinfo/' },
+    Model => { },
+    Manufacturer => { },
+    Notes => { },
+);
+my %sAppInfo = (
+    STRUCT_NAME => 'AppInfo',
+    NAMESPACE   => { AppInfo => 'http://ns.google.com/photos/dd/1.0/appinfo/' },
+    Application => { },
+    Version => { },
+    ItemURI => { },
+);
 
 # camera-raw defaults
 %Image::ExifTool::XMP::crd = (
@@ -470,16 +531,18 @@ my %sImageRegion = ( # new in 1.5
     rRole  => { Namespace => 'Iptc4xmpExt', FlatName => 'Role',  List => 'Bag', Struct => \%sEntity },
 );
 
-# IPTC Extension namespace properties (Iptc4xmpExt) (ref 4)
+# IPTC Extension namespace properties (Iptc4xmpExt) (ref 4, 14)
 %Image::ExifTool::XMP::iptcExt = (
     %xmpTableDefaults,
     GROUPS => { 1 => 'XMP-iptcExt', 2 => 'Author' },
     NAMESPACE   => 'Iptc4xmpExt',
     TABLE_DESC => 'XMP IPTC Extension',
     NOTES => q{
-        This table contains tags defined by the IPTC Extension schema version 1.5. 
-        The actual namespace prefix is "Iptc4xmpExt", but ExifTool shortens this for
-        the family 1 group name. (see L<http://www.iptc.org/IPTC4XMP/>)
+        This table contains tags defined by the IPTC Extension schema version 1.6
+        and IPTC Video Metadata version 1.3. The actual namespace prefix is
+        "Iptc4xmpExt", but ExifTool shortens this for the family 1 group name. (See
+        L<http://www.iptc.org/standards/photo-metadata/iptc-standard/> and
+        L<https://iptc.org/standards/video-metadata-hub/>.)
     },
     AboutCvTerm => {
         Struct => \%sCVTermDetails,
@@ -734,8 +797,16 @@ my %sImageRegion = ( # new in 1.5
     },
     PlanningRef         => { List => 'Bag', Struct => \%sEntityWithRole },
     audioBitsPerSample  => { Groups => { 2 => 'Audio' }, Writable => 'integer' },
+    # new IPTC video metadata 1.3 properties
+    # (ref https://iptc.org/std/videometadatahub/recommendation/IPTC-VideoMetadataHub-props-Rec_1.3.html)
+    metadataLastEdited => { Groups => { 2 => 'Time' }, %dateTimeInfo },
+    metadataLastEditor => { Struct => \%sEntity },
+    metadataAuthority  => { Struct => \%sEntity },
+    parentId           => { Name => 'ParentID' },
     # new IPTC Extension schema 1.5 property
     ImageRegion => { Groups => { 2 => 'Image' }, List => 'Bag', Struct => \%sImageRegion },
+    # new Extension 1.6 property
+    EventId     => { Name => 'EventID', List => 'Bag' },
 );
 
 #------------------------------------------------------------------------------
@@ -1167,7 +1238,7 @@ my %prismPublicationDate = (
     NAMESPACE => 'PixelLive',
     AVOID => 1,
     NOTES => q{
-        PixelLive namespace tags.  These tags are not writable becase they are very
+        PixelLive namespace tags.  These tags are not writable because they are very
         uncommon and I haven't been able to locate a reference which gives the
         namespace URI.
     },
@@ -1637,6 +1708,7 @@ my %sSubVersion = (
     InitialViewPitchDegrees         => { Writable => 'real' },
     InitialViewRollDegrees          => { Writable => 'real' },
     InitialHorizontalFOVDegrees     => { Writable => 'real' },
+    InitialVerticalFOVDegrees       => { Writable => 'real' },
     FirstPhotoDate                  => { %dateTimeInfo, Groups => { 2 => 'Time' } },
     LastPhotoDate                   => { %dateTimeInfo, Groups => { 2 => 'Time' } },
     SourcePhotosCount               => { Writable => 'integer' },
@@ -1778,6 +1850,10 @@ my %sSubVersion = (
         ValueConv => 'Image::ExifTool::XMP::DecodeBase64($val)',
         ValueConvInv => 'Image::ExifTool::XMP::EncodeBase64($val)',
     },
+    MicroVideo          => { Writable => 'integer' },
+    MicroVideoVersion   => { Writable => 'integer' },
+    MicroVideoOffset    => { Writable => 'integer' },
+    MicroVideoPresentationTimestampUs => { Writable => 'integer' },
 );
 
 # Google creations namespace (ref PH)
@@ -1789,6 +1865,158 @@ my %sSubVersion = (
     CameraBurstID  => { },
 );
 
+# Google depth-map Device namespace (ref 13)
+%Image::ExifTool::XMP::Device = (
+    %xmpTableDefaults,
+    GROUPS => { 1 => 'XMP-Device', 2 => 'Camera' },
+    NAMESPACE => { Device => 'http://ns.google.com/photos/dd/1.0/device/' },
+    NOTES => q{
+        Google depth-map Device tags.  See
+        L<https://developer.android.com/training/camera2/Dynamic-depth-v1.0.pdf> for
+        the specification.
+    },
+    Container => {
+        Struct => {
+            STRUCT_NAME => 'DeviceContainer',
+            NAMESPACE   => { Container => 'http://ns.google.com/photos/dd/1.0/container/' },
+            Directory => {
+                List => 'Seq',
+                Struct => {
+                    STRUCT_NAME => 'DeviceDirectory',
+                    NAMESPACE   => { Container => 'http://ns.google.com/photos/dd/1.0/container/' },
+                    Item => {
+                        Struct => {
+                            STRUCT_NAME => 'DeviceItem',
+                            NAMESPACE => { Item => 'http://ns.google.com/photos/dd/1.0/item/' },
+                            Mime    => { },
+                            Length  => { Writable => 'integer' },
+                            Padding => { Writable => 'integer' },
+                            DataURI => { },
+                        },
+                    },
+                },
+            }
+        },
+    },
+    Profiles => {
+        List => 'Seq',
+        FlatName => '',
+        Struct => {
+            STRUCT_NAME => 'DeviceProfiles',
+            NAMESPACE => { Device => 'http://ns.google.com/photos/dd/1.0/device/' },
+            Profile => {
+                Struct => {
+                    STRUCT_NAME => 'DeviceProfile',
+                    NAMESPACE => { Profile => 'http://ns.google.com/photos/dd/1.0/profile/' },
+                    CameraIndices => { List => 'Seq', Writable => 'integer' },
+                    Type => { },
+                },
+            },
+        },
+    },
+    Cameras => {
+        List => 'Seq',
+        FlatName => '',
+        Struct => {
+            STRUCT_NAME => 'DeviceCameras',
+            NAMESPACE => { Device => 'http://ns.google.com/photos/dd/1.0/device/' },
+            Camera => {
+                Struct => {
+                    STRUCT_NAME => 'DeviceCamera',
+                    NAMESPACE => { Camera => 'http://ns.google.com/photos/dd/1.0/camera/' },
+                    DepthMap => {
+                        Struct => {
+                            STRUCT_NAME => 'DeviceDepthMap',
+                            NAMESPACE => { DepthMap => 'http://ns.google.com/photos/dd/1.0/depthmap/' },
+                            ConfidenceURI => { },
+                            DepthURI    => { },
+                            Far         => { Writable => 'real' },
+                            Format      => { },
+                            ItemSemantic=> { },
+                            MeasureType => { },
+                            Near        => { Writable => 'real' },
+                            Units       => { },
+                            Software    => { },
+                            FocalTableEntryCount => { Writable => 'integer' },
+                            FocalTable  => { }, # (base64)
+                        },
+                    },
+                    Image => {
+                        Struct => {
+                            STRUCT_NAME => 'DeviceImage',
+                            NAMESPACE => { Image => 'http://ns.google.com/photos/dd/1.0/image/' },
+                            ItemSemantic=> { },
+                            ItemURI     => { },
+                        },
+                    },
+                    ImagingModel => {
+                        Struct => {
+                            STRUCT_NAME => 'DeviceImagingModel',
+                            NAMESPACE => { ImagingModel => 'http://ns.google.com/photos/dd/1.0/imagingmodel/' },
+                            Distortion      => { }, # (base64)
+                            DistortionCount => { Writable => 'integer' },
+                            FocalLengthX    => { Writable => 'real' },
+                            FocalLengthY    => { Writable => 'real' },
+                            ImageHeight     => { Writable => 'integer' },
+                            ImageWidth      => { Writable => 'integer' },
+                            PixelAspectRatio=> { Writable => 'real' },
+                            PrincipalPointX => { Writable => 'real' },
+                            PrincipalPointY => { Writable => 'real' },
+                            Skew            => { Writable => 'real' },
+                        },
+                    },
+                    PointCloud => {
+                        Struct => {
+                            STRUCT_NAME => 'DevicePointCloud',
+                            NAMESPACE => { PointCloud => 'http://ns.google.com/photos/dd/1.0/pointcloud/' },
+                            PointCloud  => { Writable => 'integer' },
+                            Points      => { },
+                            Metric      => { Writable => 'boolean' },
+                        },
+                    },
+                    Pose => { Struct => \%sPose },
+                    LightEstimate => {
+                        Struct => {
+                            STRUCT_NAME => 'DeviceLightEstimate',
+                            NAMESPACE => { LightEstimate => 'http://ns.google.com/photos/dd/1.0/lightestimate/' },
+                            ColorCorrectionR => { Writable => 'real' },
+                            ColorCorrectionG => { Writable => 'real' },
+                            ColorCorrectionB => { Writable => 'real' },
+                            PixelIntensity   => { Writable => 'real' },
+                        },
+                    },
+                    VendorInfo => { Struct => \%sVendorInfo },
+                    AppInfo    => { Struct => \%sAppInfo },
+                    Trait => { },
+                },
+            },
+        },
+    },
+    VendorInfo  => { Struct => \%sVendorInfo },
+    AppInfo     => { Struct => \%sAppInfo },
+    EarthPos    => { Struct => \%sEarthPose },
+    Pose        => { Struct => \%sPose },
+    Planes => {
+        List => 'Seq',
+        FlatName => '',
+        Struct => {
+            STRUCT_NAME => 'DevicePlanes',
+            NAMESPACE => { Device => 'http://ns.google.com/photos/dd/1.0/device/' },
+            Plane => {
+                Struct => {
+                    STRUCT_NAME => 'DevicePlane',
+                    NAMESPACE => { Plane => 'http://ns.google.com/photos/dd/1.0/plane/' },
+                    Pose    => { Struct => \%sPose },
+                    ExtentX => { Writable => 'real' },
+                    ExtentZ => { Writable => 'real' },
+                    BoundaryVertexCount => { Writable => 'integer' },
+                    Boundary => { },
+                },
+            },
+        },
+    },
+);
+
 # Getty Images namespace (ref PH)
 %Image::ExifTool::XMP::GettyImages = (
     %xmpTableDefaults,
@@ -1905,7 +2133,7 @@ This file contains definitions for less common XMP namespaces.
 
 =head1 AUTHOR
 
-Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
